"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const ontology_ts_crypto_1 = require("ontology-ts-crypto");
const ontology_ts_test_1 = require("ontology-ts-test");
const path = require("path");
const configLoader_1 = require("../config/configLoader");
const punicaException_1 = require("../exception/punicaException");
const fileSystem_1 = require("../utils/fileSystem");
const walletCli_1 = require("../wallet/walletCli");
// tslint:disable:no-console
class Deployer {
    async deploy(projectDir, networkKey, avmFileName, walletFileName, configKey) {
        let avmDirPath;
        if (avmFileName !== undefined) {
            const avmPath = path.join(projectDir, avmFileName);
            if (fs.existsSync(avmPath)) {
                avmDirPath = path.dirname(avmPath);
                avmFileName = path.basename(avmPath);
            }
            else {
                avmDirPath = path.join(projectDir, 'contracts', 'build');
            }
        }
        else {
            avmDirPath = path.join(projectDir, 'contracts', 'build');
        }
        if (!fs.existsSync(avmDirPath)) {
            // tslint:disable-next-line:no-console
            throw punicaException_1.otherError(`${avmDirPath} does not exist.`);
        }
        const rpcAddress = configLoader_1.loadNetwork(projectDir, networkKey);
        const deployInfo = configLoader_1.loadDeploy(projectDir, configKey);
        const [wallet] = configLoader_1.loadWallet(projectDir, walletFileName);
        const account = configLoader_1.loadAccount(wallet, deployInfo.payer);
        const configPassword = configLoader_1.loadPassword(projectDir, configKey, account.address.toBase58());
        let avm;
        [avm, avmFileName] = fileSystem_1.readAvm(avmDirPath, avmFileName);
        const client = ontology_ts_test_1.initClient({ rpcAddress });
        console.log(`Running deployment: ${avmFileName}`);
        // test if contract is already deployed
        const contractAddress = this.generateContractAddress(avm);
        const deployed = await ontology_ts_test_1.isDeployed({ client, scriptHash: contractAddress });
        if (deployed) {
            console.log(`Contract is already deployed at 0x${contractAddress}`);
            return;
        }
        let password;
        if (configPassword !== undefined) {
            password = configPassword;
        }
        else {
            password = await walletCli_1.inputExistingPassword('Please input payer account password: ');
        }
        // try the password
        await account.decryptKey(password);
        await ontology_ts_test_1.deploy({
            client,
            account,
            password,
            code: avm,
            gasLimit: String(deployInfo.gasLimit),
            gasPrice: String(deployInfo.gasPrice),
            name: deployInfo.name,
            version: deployInfo.version,
            author: deployInfo.author,
            email: deployInfo.email,
            description: deployInfo.desc,
            needStorage: deployInfo.needStorage
        });
        console.log(`Contract has been deployed to address 0x${contractAddress}.`);
    }
    generateContractAddress(avm) {
        return ontology_ts_test_1.reverseBuffer(ontology_ts_crypto_1.Address.fromVmCode(avm).toArray()).toString('hex');
    }
}
exports.Deployer = Deployer;
//# sourceMappingURL=deployer.js.map